.listtagged li.header { font-weight: bold; text-decoration: underline; }

.listtags .listtagged li.header { font-weight: normal; } 
.listtags .listtagged ul { margin: 0px; padding: 0px; }

#content ul.tags {
	float: right;
	list-style: none;
	text-align: center;
	max-width: 20em;
}
#content .tags li { display: inline; text-align: center; }

#content .tags li.header {
	font-weight: bold;
	font-size: large;
	text-align: center;
	color: #ccc;
	text-transform: lowercase;
	letter-spacing: 0.5em;
	display: block;
}

ul.tagcloud { list-style: none; }
.tagcloud li { display: inline; }
.tagcloud li:after { content: ", "; }
.tagcloud li.last:after { content: ""; }

#query {
	float: right;
	clear: both;
	font-size: 10px;
	text-align: left;
}
#query ul {
    list-style: none;
    padding: 0;
    padding-left: 2em;
}
#query input { font-size: 10px }
#query-error {
	color: #a00;
	font-size: 11px;
	margin: 0.3em 0;
}
